@@ -1,5 +1,5 @@
 # IMAP server support
-# Copyright (C) 2002 - 2011 John Goerzen & contributors
+# Copyright (C) 2002-2016 John Goerzen & contributors.
 #
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -15,23 +15,23 @@
 #    along with this program; if not, write to the Free Software
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-from threading import Lock, BoundedSemaphore, Thread, Event, currentThread
 import hmac
 import socket
 import base64
-
 import json
 import urllib
-
-import socket
 import time
 import errno
-from sys import exc_info
+import socket
 from socket import gaierror
+from sys import exc_info
 from ssl import SSLError, cert_time_to_seconds
+from threading import Lock, BoundedSemaphore, Thread, Event, currentThread
+
+import six
 
-from offlineimap import imaplibutil, imaputil, threadutil, OfflineImapError
 import offlineimap.accounts
+from offlineimap import imaplibutil, imaputil, threadutil, OfflineImapError
 from offlineimap.ui import getglobalui
 
 
@@ -44,7 +44,8 @@ try:
 except ImportError:
     pass
 
-class IMAPServer:
+
+class IMAPServer(object):
     """Initializes all variables from an IMAPRepository() instance
 
     Various functions, such as acquireconnection() return an IMAP4
@@ -56,7 +57,10 @@ class IMAPServer:
 
     GSS_STATE_STEP = 0
     GSS_STATE_WRAP = 1
+
     def __init__(self, repos):
+        """:repos: a IMAPRepository instance."""
+
         self.ui = getglobalui()
         self.repos = repos
         self.config = repos.getconfig()
@@ -82,25 +86,30 @@ class IMAPServer:
 
         self.usessl = repos.getssl()
         self.useipv6 = repos.getipv6()
-        if self.useipv6 == True:
+        if self.useipv6 is True:
             self.af = socket.AF_INET6
-        elif self.useipv6 == False:
+        elif self.useipv6 is False:
             self.af = socket.AF_INET
         else:
             self.af = socket.AF_UNSPEC
-        self.hostname = \
-            None if self.preauth_tunnel else repos.gethost()
+        self.hostname = None if self.transport_tunnel or self.preauth_tunnel else repos.gethost()
         self.port = repos.getport()
-        if self.port == None:
+        if self.port is None:
             self.port = 993 if self.usessl else 143
         self.sslclientcert = repos.getsslclientcert()
         self.sslclientkey = repos.getsslclientkey()
         self.sslcacertfile = repos.getsslcacertfile()
         if self.sslcacertfile is None:
-            self.__verifycert = None # disable cert verification
+            self.__verifycert = None # Disable cert verification.
+                                     # This way of working sucks hard...
         self.fingerprint = repos.get_ssl_fingerprint()
-        self.sslversion = repos.getsslversion()
         self.tlslevel = repos.gettlslevel()
+        self.sslversion = repos.getsslversion()
+        self.starttls = repos.getstarttls()
+
+        if self.tlslevel is not "tls_compat" and self.sslversion is None:
+            raise Exception("When 'tls_version' is not 'tls_compat' "
+                "the 'ssl_version' must be set explicitly.")
 
         self.oauth2_refresh_token = repos.getoauth2_refresh_token()
         self.oauth2_access_token = repos.getoauth2_access_token()
@@ -125,30 +134,47 @@ class IMAPServer:
         # In order to support proxy connection, we have to override the
         # default socket instance with our own socksified socket instance.
         # We add this option to bypass the GFW in China.
-        _account_section = 'Account ' + self.repos.account.name
-        if not self.config.has_option(_account_section, 'proxy'):
-            self.proxied_socket = socket.socket
-        else:
-            proxy = self.config.get(_account_section, 'proxy')
-            # Powered by PySocks.
-            try:
-                import socks
-                proxy_type, host, port = proxy.split(":")
-                port = int(port)
-                socks.setdefaultproxy(getattr(socks, proxy_type), host, port)
-                self.proxied_socket = socks.socksocket
-            except ImportError:
-                self.ui.warn("PySocks not installed, ignoring proxy option.")
-                self.proxied_socket = socket.socket
-            except (AttributeError, ValueError) as e:
-                self.ui.warn("Bad proxy option %s for account %s: %s "
-                    "Ignoring proxy option."%
-                    (proxy, self.repos.account.name, e))
-                self.proxied_socket = socket.socket
+        self.proxied_socket = self._get_proxy('proxy', socket.socket)
 
+        # Turns out that the GFW in China is no longer blocking imap.gmail.com
+        # However accounts.google.com (for oauth2) definitey is.  Therefore
+        # it is not strictly necessary to use a proxy for *both* IMAP *and*
+        # oauth2, so a new option is added: authproxy.
+
+        # Set proxy for use in authentication (only) if desired.
+        # If not set, is same as proxy option (compatible with current configs)
+        # To use a proxied_socket but not an authproxied_socket
+        # set authproxy = '' in config
+        self.authproxied_socket = self._get_proxy('authproxy',
+                                                  self.proxied_socket)
+
+    def _get_proxy(self, proxysection, dfltsocket):
+        _account_section = 'Account ' + self.repos.account.name
+        if not self.config.has_option(_account_section, proxysection):
+            return dfltsocket
+        proxy = self.config.get(_account_section, proxysection)
+        if proxy == '':
+            # explicitly set no proxy (overrides default return of dfltsocket)
+            return socket.socket
+
+        # Powered by PySocks.
+        try:
+            import socks
+            proxy_type, host, port = proxy.split(":")
+            port = int(port)
+            socks.setdefaultproxy(getattr(socks, proxy_type), host, port)
+            return socks.socksocket
+        except ImportError:
+            self.ui.warn("PySocks not installed, ignoring proxy option.")
+        except (AttributeError, ValueError) as e:
+            self.ui.warn("Bad proxy option %s for account %s: %s "
+                "Ignoring %s option."%
+                (proxy, self.repos.account.name, e, proxysection))
+        return dfltsocket
 
     def __getpassword(self):
         """Returns the server password or None"""
+
         if self.goodpassword != None: # use cached good one first
             return self.goodpassword
 
@@ -162,31 +188,6 @@ class IMAPServer:
         self.passworderror = None
         return self.password
 
-    # XXX: is this function used anywhere?
-    def getroot(self):
-        """Returns this server's folder root. Can only be called after one
-        or more calls to acquireconnection."""
-
-        return self.root
-
-
-    def releaseconnection(self, connection, drop_conn=False):
-        """Releases a connection, returning it to the pool.
-
-        :param drop_conn: If True, the connection will be released and
-           not be reused. This can be used to indicate broken connections."""
-
-        if connection is None: return #noop on bad connection
-        self.connectionlock.acquire()
-        self.assignedconnections.remove(connection)
-        # Don't reuse broken connections
-        if connection.Terminate or drop_conn:
-            connection.logout()
-        else:
-            self.availableconnections.append(connection)
-        self.connectionlock.release()
-        self.semaphore.release()
-
     def __md5handler(self, response):
         challenge = response.strip()
         self.ui.debug('imap', '__md5handler: got challenge %s'% challenge)
@@ -202,54 +203,68 @@ class IMAPServer:
         self.ui.debug('imap', 'Attempting IMAP LOGIN authentication')
         imapobj.login(self.username, self.__getpassword())
 
-
     def __plainhandler(self, response):
         """Implements SASL PLAIN authentication, RFC 4616,
           http://tools.ietf.org/html/rfc4616"""
 
         authc = self.username
         passwd = self.__getpassword()
-        authz = ''
+        authz = b''
         if self.user_identity != None:
             authz = self.user_identity
-        NULL = u'\x00'
-        retval = NULL.join((authz, authc, passwd)).encode('utf-8')
-        logsafe_retval = NULL.join((authz, authc, "(passwd hidden for log)")).encode('utf-8')
-        self.ui.debug('imap', '__plainhandler: returning %s' % logsafe_retval)
+        # At this point all authz, authc and passwd are expected bytes encoded
+        # in UTF-8.
+        NULL = b'\x00'
+        retval = NULL.join((authz, authc, passwd))
+        logsafe_retval = NULL.join((authz, authc, b'(passwd hidden for log)'))
+        self.ui.debug('imap', '__plainhandler: returning %s'% logsafe_retval)
         return retval
 
-
     def __xoauth2handler(self, response):
-        if self.oauth2_refresh_token is None and self.oauth2_access_token is None:
-            return None
-
         if self.oauth2_access_token is None:
-            # need to move these to config
-            # generate new access token
+            if self.oauth2_request_url is None:
+                raise OfflineImapError("No remote oauth2_request_url for "
+                    "repository '%s' specified."%
+                    self, OfflineImapError.ERROR.REPO)
+
+            # Generate new access token.
             params = {}
             params['client_id'] = self.oauth2_client_id
             params['client_secret'] = self.oauth2_client_secret
             params['refresh_token'] = self.oauth2_refresh_token
             params['grant_type'] = 'refresh_token'
 
-            self.ui.debug('imap', 'xoauth2handler: url "%s"' % self.oauth2_request_url)
-            self.ui.debug('imap', 'xoauth2handler: params "%s"' % params)
+            self.ui.debug('imap', 'xoauth2handler: url "%s"'%
+                self.oauth2_request_url)
+            self.ui.debug('imap', 'xoauth2handler: params "%s"'% params)
 
             original_socket = socket.socket
-            socket.socket = self.proxied_socket
+            socket.socket = self.authproxied_socket
             try:
-                response = urllib.urlopen(self.oauth2_request_url, urllib.urlencode(params)).read()
+                response = urllib.urlopen(
+                    self.oauth2_request_url, urllib.urlencode(params)).read()
+            except Exception as e:
+                try:
+                    msg = "%s (configuration is: %s)"% (e, str(params))
+                except Exception as eparams:
+                    msg = "%s [cannot display configuration: %s]"% (e, eparams)
+                six.reraise(type(e), type(e)(msg), exc_info()[2])
             finally:
                 socket.socket = original_socket
 
             resp = json.loads(response)
-            self.ui.debug('imap', 'xoauth2handler: response "%s"' % resp)
+            self.ui.debug('imap', 'xoauth2handler: response "%s"'% resp)
+            if u'error' in resp:
+                raise OfflineImapError("xoauth2handler got: %s"% resp,
+                    OfflineImapError.ERROR.REPO)
             self.oauth2_access_token = resp['access_token']
 
-        self.ui.debug('imap', 'xoauth2handler: access_token "%s"' % self.oauth2_access_token)
-        auth_string = 'user=%s\1auth=Bearer %s\1\1' % (self.username, self.oauth2_access_token)
+        self.ui.debug('imap', 'xoauth2handler: access_token "%s"'%
+            self.oauth2_access_token)
+        auth_string = 'user=%s\1auth=Bearer %s\1\1'% (
+            self.username, self.oauth2_access_token)
         #auth_string = base64.b64encode(auth_string)
-        self.ui.debug('imap', 'xoauth2handler: returning "%s"' % auth_string)
+        self.ui.debug('imap', 'xoauth2handler: returning "%s"'% auth_string)
         return auth_string
 
     def __gssauth(self, response):
@@ -279,7 +294,6 @@ class IMAPServer:
             response = ''
         return base64.b64decode(response)
 
-
     def __start_tls(self, imapobj):
         if 'STARTTLS' in imapobj.capabilities and not self.usessl:
             self.ui.debug('imap', 'Using STARTTLS connection')
@@ -290,7 +304,6 @@ class IMAPServer:
                     "TLS connection: %s"% str(e),
                     OfflineImapError.ERROR.REPO, None, exc_info()[2])
 
-
     ## All __authn_* procedures are helpers that do authentication.
     ## They are class methods that take one parameter, IMAP object.
     ##
@@ -336,6 +349,10 @@ class IMAPServer:
         return True
 
     def __authn_xoauth2(self, imapobj):
+        if self.oauth2_refresh_token is None \
+                and self.oauth2_access_token is None:
+            return False
+
         imapobj.authenticate('XOAUTH2', self.__xoauth2handler)
         return True
 
@@ -350,7 +367,6 @@ class IMAPServer:
             self.__loginauth(imapobj)
             return True
 
-
     def __authn_helper(self, imapobj):
         """Authentication machinery for self.acquireconnection().
 
@@ -362,6 +378,10 @@ class IMAPServer:
         warnings for failed methods are to be produced in the
         respective except blocks."""
 
+        # Stack stores pairs of (method name, exception)
+        exc_stack = []
+        tried_to_authn = False
+        tried_tls = False
         # Authentication routines, hash keyed by method name
         # with value that is a tuple with
         # - authentication function,
@@ -369,24 +389,15 @@ class IMAPServer:
         # - check IMAP capability flag.
         auth_methods = {
           "GSSAPI": (self.__authn_gssapi, False, True),
-          "CRAM-MD5": (self.__authn_cram_md5, True, True),
           "XOAUTH2": (self.__authn_xoauth2, True, True),
+          "CRAM-MD5": (self.__authn_cram_md5, True, True),
           "PLAIN": (self.__authn_plain, True, True),
           "LOGIN": (self.__authn_login, True, False),
         }
-        # Stack stores pairs of (method name, exception)
-        exc_stack = []
-        tried_to_authn = False
-        tried_tls = False
-        mechs = self.authmechs
 
-        # GSSAPI must be tried first: we will probably go TLS after it
-        # and GSSAPI mustn't be tunneled over TLS.
-        if "GSSAPI" in mechs:
-            mechs.remove("GSSAPI")
-            mechs.insert(0, "GSSAPI")
-
-        for m in mechs:
+        # GSSAPI is tried first by default: we will probably go TLS after it and
+        # GSSAPI mustn't be tunneled over TLS.
+        for m in self.authmechs:
             if m not in auth_methods:
                 raise Exception("Bad authentication method %s, "
                   "please, file OfflineIMAP bug" % m)
@@ -395,7 +406,7 @@ class IMAPServer:
 
             # TLS must be initiated before checking capabilities:
             # they could have been changed after STARTTLS.
-            if tryTLS and not tried_tls:
+            if tryTLS and self.starttls and not tried_tls:
                 tried_tls = True
                 self.__start_tls(imapobj)
 
@@ -415,26 +426,58 @@ class IMAPServer:
                 exc_stack.append((m, e))
 
         if len(exc_stack):
-            msg = "\n\t".join(map(
-              lambda x: ": ".join((x[0], str(x[1]))),
-              exc_stack
-            ))
+            msg = "\n\t".join([": ".join((x[0], str(x[1]))) for x in exc_stack])
             raise OfflineImapError("All authentication types "
               "failed:\n\t%s"% msg, OfflineImapError.ERROR.REPO)
 
         if not tried_to_authn:
-            methods = ", ".join(map(
-              lambda x: x[5:], filter(lambda x: x[0:5] == "AUTH=",
-               imapobj.capabilities)
-            ))
+            methods = ", ".join([x[5:] for x in
+                [x for x in imapobj.capabilities if x[0:5] == "AUTH="]])
             raise OfflineImapError(u"Repository %s: no supported "
               "authentication mechanisms found; configured %s, "
               "server advertises %s"% (self.repos,
               ", ".join(self.authmechs), methods),
               OfflineImapError.ERROR.REPO)
 
+    def __verifycert(self, cert, hostname):
+        """Verify that cert (in socket.getpeercert() format) matches hostname.
+
+        CRLs are not handled.
+        Returns error message if any problems are found and None on success."""
+
+        errstr = "CA Cert verifying failed: "
+        if not cert:
+            return ('%s no certificate received'% errstr)
+        dnsname = hostname.lower()
+        certnames = []
+
+        # cert expired?
+        notafter = cert.get('notAfter')
+        if notafter:
+            if time.time() >= cert_time_to_seconds(notafter):
+                return '%s certificate expired %s'% (errstr, notafter)
+
+        # First read commonName
+        for s in cert.get('subject', []):
+            key, value = s[0]
+            if key == 'commonName':
+                certnames.append(value.lower())
+        if len(certnames) == 0:
+            return ('%s no commonName found in certificate'% errstr)
+
+        # Then read subjectAltName
+        for key, value in cert.get('subjectAltName', []):
+            if key == 'DNS':
+                certnames.append(value.lower())
+
+        # And finally try to match hostname with one of these names
+        for certname in certnames:
+            if (certname == dnsname or
+                '.' in dnsname and certname == '*.' + dnsname.split('.', 1)[1]):
+                return None
+
+        return ('%s no matching domain name found in certificate'% errstr)
 
-    # XXX: move above, closer to releaseconnection()
     def acquireconnection(self):
         """Fetches a connection from the pool, making sure to create a new one
         if needed, to obey the maximum connection limits, etc.
@@ -450,7 +493,6 @@ class IMAPServer:
             # Try to find one that previously belonged to this thread
             # as an optimization.  Start from the back since that's where
             # they're popped on.
-            imapobj = None
             for i in range(len(self.availableconnections) - 1, -1, -1):
                 tryobj = self.availableconnections[i]
                 if self.lastowner[tryobj] == curThread.ident:
@@ -469,28 +511,32 @@ class IMAPServer:
 
         # Must be careful here that if we fail we should bail out gracefully
         # and release locks / threads so that the next attempt can try...
-        success = 0
+        success = False
         try:
-            while not success:
+            while success is not True:
                 # Generate a new connection.
                 if self.tunnel:
-                    self.ui.connecting('tunnel', self.tunnel)
+                    self.ui.connecting(
+                        self.repos.getname(), 'tunnel', self.tunnel)
                     imapobj = imaplibutil.IMAP4_Tunnel(
                         self.tunnel,
                         timeout=socket.getdefaulttimeout(),
                         use_socket=self.proxied_socket,
                         )
-                    success = 1
+                    success = True
                 elif self.usessl:
-                    self.ui.connecting(self.hostname, self.port)
+                    self.ui.connecting(
+                        self.repos.getname(), self.hostname, self.port)
+                    self.ui.debug('imap', "%s: level '%s', version '%s'"%
+                        (self.repos.getname(), self.tlslevel, self.sslversion))
                     imapobj = imaplibutil.WrappedIMAP4_SSL(
-                        self.hostname,
-                        self.port,
-                        self.sslclientkey,
-                        self.sslclientcert,
-                        self.sslcacertfile,
-                        self.__verifycert,
-                        self.sslversion,
+                        host=self.hostname,
+                        port=self.port,
+                        keyfile=self.sslclientkey,
+                        certfile=self.sslclientcert,
+                        ca_certs=self.sslcacertfile,
+                        cert_verify_cb=self.__verifycert,
+                        ssl_version=self.sslversion,
                         timeout=socket.getdefaulttimeout(),
                         fingerprint=self.fingerprint,
                         use_socket=self.proxied_socket,
@@ -498,7 +544,8 @@ class IMAPServer:
                         af=self.af,
                         )
                 else:
-                    self.ui.connecting(self.hostname, self.port)
+                    self.ui.connecting(
+                        self.repos.getname(), self.hostname, self.port)
                     imapobj = imaplibutil.WrappedIMAP4(
                         self.hostname, self.port,
                         timeout=socket.getdefaulttimeout(),
@@ -510,7 +557,7 @@ class IMAPServer:
                     try:
                         self.__authn_helper(imapobj)
                         self.goodpassword = self.password
-                        success = 1
+                        success = True
                     except OfflineImapError as e:
                         self.passworderror = str(e)
                         raise
@@ -562,7 +609,9 @@ class IMAPServer:
                          "'%s'. Make sure you have configured the ser"\
                          "ver name correctly and that you are online."%\
                          (self.hostname, self.repos)
-                raise OfflineImapError(reason, severity), None, exc_info()[2]
+                six.reraise(OfflineImapError,
+                            OfflineImapError(reason, severity),
+                            exc_info()[2])
 
             elif isinstance(e, SSLError) and e.errno == errno.EPERM:
                 # SSL unknown protocol error
@@ -570,12 +619,15 @@ class IMAPServer:
                 if self.port != 993:
                     reason = "Could not connect via SSL to host '%s' and non-s"\
                         "tandard ssl port %d configured. Make sure you connect"\
-                        " to the correct port."% (self.hostname, self.port)
+                        " to the correct port. Got: %s"% (
+                            self.hostname, self.port, e)
                 else:
                     reason = "Unknown SSL protocol connecting to host '%s' for "\
                          "repository '%s'. OpenSSL responded:\n%s"\
                          % (self.hostname, self.repos, e)
-                raise OfflineImapError(reason, severity), None, exc_info()[2]
+                six.reraise(OfflineImapError,
+                            OfflineImapError(reason, severity),
+                            exc_info()[2])
 
             elif isinstance(e, socket.error) and e.args[0] == errno.ECONNREFUSED:
                 # "Connection refused", can be a non-existing port, or an unauthorized
@@ -584,14 +636,19 @@ class IMAPServer:
                     "refused. Make sure you have the right host and port "\
                     "configured and that you are actually able to access the "\
                     "network."% (self.hostname, self.port, self.repos)
-                raise OfflineImapError(reason, severity), None, exc_info()[2]
+                six.reraise(OfflineImapError,
+                            OfflineImapError(reason, severity),
+                            exc_info()[2])
             # Could not acquire connection to the remote;
             # socket.error(last_error) raised
             if str(e)[:24] == "can't open socket; error":
-                raise OfflineImapError("Could not connect to remote server '%s' "\
-                    "for repository '%s'. Remote does not answer."
-                    % (self.hostname, self.repos),
-                    OfflineImapError.ERROR.REPO), None, exc_info()[2]
+                six.reraise(OfflineImapError,
+                            OfflineImapError(
+                                "Could not connect to remote server '%s' "
+                                "for repository '%s'. Remote does not answer."%
+                                (self.hostname, self.repos),
+                                OfflineImapError.ERROR.REPO),
+                            exc_info()[2])
             else:
                 # re-raise all other errors
                 raise
@@ -606,7 +663,7 @@ class IMAPServer:
         It's OK if we have maxconnections + 1 or 2 threads, which is what this
         will help us do."""
 
-        self.semaphore.acquire()
+        self.semaphore.acquire() # Blocking until maxconnections has free slots.
         self.semaphore.release()
 
     def close(self):
@@ -645,7 +702,8 @@ class IMAPServer:
 
             threads = []
             for i in range(numconnections):
-                self.ui.debug('imap', 'keepalive: processing connection %d of %d'% (i, numconnections))
+                self.ui.debug('imap', 'keepalive: processing connection %d of %d'%
+                                      (i, numconnections))
                 if len(self.idlefolders) > i:
                     # IDLE thread
                     idler = IdleThread(self, self.idlefolders[i])
@@ -667,44 +725,25 @@ class IMAPServer:
         self.ui.debug('imap', 'keepalive: event is set; exiting')
         return
 
-    def __verifycert(self, cert, hostname):
-        """Verify that cert (in socket.getpeercert() format) matches hostname.
 
-        CRLs are not handled.
-        Returns error message if any problems are found and None on success."""
+    def releaseconnection(self, connection, drop_conn=False):
+        """Releases a connection, returning it to the pool.
 
-        errstr = "CA Cert verifying failed: "
-        if not cert:
-            return ('%s no certificate received'% errstr)
-        dnsname = hostname.lower()
-        certnames = []
+        :param drop_conn: If True, the connection will be released and
+           not be reused. This can be used to indicate broken connections."""
 
-        # cert expired?
-        notafter = cert.get('notAfter')
-        if notafter:
-            if time.time() >= cert_time_to_seconds(notafter):
-                return '%s certificate expired %s'% (errstr, notafter)
+        if connection is None:
+            return # Noop on bad connection.
 
-        # First read commonName
-        for s in cert.get('subject', []):
-            key, value = s[0]
-            if key == 'commonName':
-                certnames.append(value.lower())
-        if len(certnames) == 0:
-            return ('%s no commonName found in certificate'% errstr)
-
-        # Then read subjectAltName
-        for key, value in cert.get('subjectAltName', []):
-            if key == 'DNS':
-                certnames.append(value.lower())
-
-        # And finally try to match hostname with one of these names
-        for certname in certnames:
-            if (certname == dnsname or
-                '.' in dnsname and certname == '*.' + dnsname.split('.', 1)[1]):
-                return None
-
-        return ('%s no matching domain name found in certificate'% errstr)
+        self.connectionlock.acquire()
+        self.assignedconnections.remove(connection)
+        # Don't reuse broken connections
+        if connection.Terminate or drop_conn:
+            connection.logout()
+        else:
+            self.availableconnections.append(connection)
+        self.connectionlock.release()
+        self.semaphore.release()
 
 
 class IdleThread(object):
@@ -758,7 +797,7 @@ class IdleThread(object):
         localrepos = account.localrepos
         remoterepos = account.remoterepos
         statusrepos = account.statusrepos
-        remotefolder = remoterepos.getfolder(self.folder)
+        remotefolder = remoterepos.getfolder(self.folder, decode=False)
 
         hook = account.getconf('presynchook', '')
         account.callhook(hook)
@@ -779,12 +818,27 @@ class IdleThread(object):
             while in IDLE mode, b) we get an Exception (e.g. on dropped
             connections, or c) the standard imaplib IDLE timeout of 29
             minutes kicks in."""
+
             result, cb_arg, exc_data = args
             if exc_data is None and not self.stop_sig.isSet():
                 # No Exception, and we are not supposed to stop:
                 self.needsync = True
             self.stop_sig.set() # Continue to sync.
 
+        def noop(imapobj):
+            """Factorize the noop code."""
+
+            try:
+                # End IDLE mode with noop, imapobj can point to a dropped conn.
+                imapobj.noop()
+            except imapobj.abort:
+                self.ui.warn('Attempting NOOP on dropped connection %s'%
+                    imapobj.identifier)
+                self.parent.releaseconnection(imapobj, True)
+            else:
+                self.parent.releaseconnection(imapobj)
+
+
         while not self.stop_sig.isSet():
             self.needsync = False
 
@@ -811,17 +865,9 @@ class IdleThread(object):
             else:
                 self.ui.warn("IMAP IDLE not supported on server '%s'."
                     "Sleep until next refresh cycle."% imapobj.identifier)
-                imapobj.noop()
+                noop(imapobj) #XXX: why?
             self.stop_sig.wait() # self.stop() or IDLE callback are invoked.
-            try:
-                # End IDLE mode with noop, imapobj can point to a dropped conn.
-                imapobj.noop()
-            except imapobj.abort:
-                self.ui.warn('Attempting NOOP on dropped connection %s'%
-                    imapobj.identifier)
-                self.parent.releaseconnection(imapobj, True)
-            else:
-                self.parent.releaseconnection(imapobj)
+            noop(imapobj)
 
             if self.needsync:
                 # Here not via self.stop, but because IDLE responded. Do
